@import "@patternfly/react-styles/css/components/Consoles/SerialConsole.css";

.pf-c-console,
.pf-c-console__vnc,
.pf-c-console__vnc > div,
.pf-c-console__vnc > div > div {
    width: 100%;
    height: 100%;
}

.pf-c-console__actions {
    max-width: 25ch;
}

.pf-c-console {
     /* auto minimum width, causing problem */
     grid-template-columns: 1fr;
     grid-template-rows: min-content 1fr;
}

.consoles-page-expanded .pf-c-page__main {
    .actions-pagesection {
        padding-bottom: 0;
    }
}

/* Hide send key button - there is not way to do that from the JS
 * https://github.com/patternfly/patternfly-react/issues/3689
 */
#pf-c-console__send-shortcut {
    display: none;
}
